
# q-expansion of newform 2304.3.e.m, downloaded from the LMFDB on 26 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2304
weight = 3
poly_data = [256, 0, 0, 0, 49, 0, 0, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 0, 65, 0, 0, 0, 1, 0], 36], [[0, -464, 0, -181, 0, -16, 0, -5], 576], [[0, 0, -33, 0, 0, 0, -1, 0], 4], [[98, 0, 0, 0, 4, 0, 0, 0], 9], [[0, 464, 0, -181, 0, 16, 0, -5], 288], [[0, -1616, 0, -701, 0, -16, 0, -13], 576], [[0, 1616, 0, -701, 0, 16, 0, -13], 144]]

hecke_ring_character_values = [[1279, [1, 0, 0, 0, 0, 0, 0, 0]], [2053, [1, 0, 0, 0, 0, 0, 0, 0]], [1793, [-1, 0, 0, 0, 0, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 0, 0, 1, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, -1, 0, 0, 0], [0, 0, -5, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 1], [0, 5, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 5, 0], [0, 0, 0, 0, 0, -15, 0, 0], [0, 0, 0, 0, -6, 0, 0, 0], [0, 0, -19, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, -11, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 5, 0], [0, 0, 0, 4, 0, 0, 0, 0], [0, 0, 0, 0, -10, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, 25, 0, 0, 0, 0, 0, 0], [-40, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -45, 0, 0], [0, 0, 0, -5, 0, 0, 0, 0], [0, 0, -81, 0, 0, 0, 0, 0], [-40, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -5, 0], [0, 0, 0, 0, 0, 59, 0, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 25, 0, 0, 0], [0, 0, -145, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -51, 0, 0], [0, 0, 0, 6, 0, 0, 0, 0], [0, 0, -85, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 5], [0, 0, 0, 0, 0, 0, -25, 0], [0, 0, 0, 0, 0, 45, 0, 0], [0, 0, 0, 0, -6, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 10], [0, 40, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -55, 0], [0, 0, 0, 6, 0, 0, 0, 0], [0, 0, 0, 0, 25, 0, 0, 0], [0, 40, 0, 0, 0, 0, 0, 0], [-150, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 27, 0], [0, 0, 0, 0, 0, -25, 0, 0], [0, 0, 0, 0, 0, 0, 0, -15], [0, 0, 0, 0, 0, 1, 0, 0], [0, 0, 0, -15, 0, 0, 0, 0], [0, 0, 0, 0, -25, 0, 0, 0], [0, 0, -135, 0, 0, 0, 0, 0], [0, 10, 0, 0, 0, 0, 0, 0], [-240, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 21, 0, 0, 0, 0], [0, 0, -15, 0, 0, 0, 0, 0], [0, -90, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 75, 0], [0, 0, 0, 0, 0, 85, 0, 0], [0, 0, 0, 0, -39, 0, 0, 0], [0, 0, -161, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 20], [0, 0, 0, 0, 0, 0, 37, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, 10, 0, 0, 0, 0, 0, 0], [-250, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 3, 0], [0, 0, 0, 0, 0, 0, 0, 1], [-520, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -15, 0, 0, 0, 0], [0, 0, 0, 0, -10, 0, 0, 0], [0, 0, 255, 0, 0, 0, 0, 0], [0, -85, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 129, 0, 0], [0, 0, 0, 0, 6, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, -46, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -65, 0], [0, 0, 0, 0, -66, 0, 0, 0], [0, 0, -189, 0, 0, 0, 0, 0], [-232, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -1, 0, 0, 0, 0], [0, 0, 0, 0, -15, 0, 0, 0], [0, -35, 0, 0, 0, 0, 0, 0], [-190, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -235, 0, 0], [0, 0, 0, -5, 0, 0, 0, 0], [0, 0, 309, 0, 0, 0, 0, 0], [600, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 15, 0], [0, 0, 0, 0, 0, 19, 0, 0], [0, 0, 0, 5, 0, 0, 0, 0], [0, 215, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 11, 0, 0], [0, 0, 0, 26, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 4], [0, 139, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -115, 0], [0, 0, 651, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 25], [0, 0, 0, 0, 105, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -40], [0, 0, 0, 0, 0, 0, 15, 0], [0, 0, 0, -65, 0, 0, 0, 0], [0, 0, 51, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -34], [890, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 60, 0, 0, 0, 0], [0, 0, -335, 0, 0, 0, 0, 0], [0, 185, 0, 0, 0, 0, 0, 0], [310, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -131, 0, 0], [0, 0, 0, 0, -34, 0, 0, 0], [0, 0, 415, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 30], [0, 0, 0, 0, 0, 135, 0, 0], [0, 0, 851, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 10], [0, 211, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 177, 0], [0, 0, 0, -16, 0, 0, 0, 0], [0, 0, 0, 0, -110, 0, 0, 0], [1070, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -7, 0], [0, 0, 0, -65, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 14], [0, 0, 0, 0, 0, 0, 85, 0], [0, 0, 0, 0, 15, 0, 0, 0], [0, -75, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 459, 0, 0], [0, 0, 0, 0, -9, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 6], [0, 316, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -235, 0, 0], [0, 0, 0, 0, -89, 0, 0, 0], [0, 0, -51, 0, 0, 0, 0, 0], [-10, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -43, 0], [0, 0, 0, 0, 0, 0, 0, 35], [0, 0, 0, 0, 0, 0, -55, 0], [0, 0, 389, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -5], [0, 0, 0, 0, 0, 0, 35, 0], [0, 0, 0, 0, 0, 509, 0, 0], [0, 0, 0, 30, 0, 0, 0, 0], [0, 0, 0, 0, -135, 0, 0, 0], [0, -205, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -134, 0, 0, 0], [0, 0, 525, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -40], [0, -340, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 46, 0, 0, 0], [0, 0, -79, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -60], [0, 210, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -40], [0, 140, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -275, 0, 0], [0, 0, 1059, 0, 0, 0, 0, 0], [510, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 95, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 0, -645, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -101, 0, 0], [0, 0, 0, 39, 0, 0, 0, 0], [0, 0, -275, 0, 0, 0, 0, 0], [0, -246, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -315, 0, 0], [0, 0, 0, 0, -114, 0, 0, 0], [-200, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -255, 0], [0, 0, 0, -54, 0, 0, 0, 0], [0, 0, 0, 0, 10, 0, 0, 0], [0, 250, 0, 0, 0, 0, 0, 0], [1360, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 255, 0, 0], [0, 0, -1, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 10], [0, 0, 0, 0, 0, 0, -185, 0], [0, 0, 0, 0, 0, 361, 0, 0], [0, 0, 0, 0, 210, 0, 0, 0], [0, 0, 0, 0, 0, 399, 0, 0], [0, 0, 0, -111, 0, 0, 0, 0], [0, 0, 0, 0, -81, 0, 0, 0], [0, 0, 255, 0, 0, 0, 0, 0], [0, -286, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 225, 0], [0, 0, 0, 0, -129, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 50], [-1440, 0, 0, 0, 0, 0, 0, 0], [0, -70, 0, 0, 0, 0, 0, 0], [-160, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -10, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 70], [0, 0, 0, 0, 0, 0, 55, 0], [0, 0, 0, 130, 0, 0, 0, 0], [0, 0, -1135, 0, 0, 0, 0, 0], [-1798, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -89, 0, 0, 0], [0, 0, -665, 0, 0, 0, 0, 0], [0, -76, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -95, 0], [0, 0, 0, 0, 0, -75, 0, 0], [0, 0, 0, 0, 41, 0, 0, 0], [680, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, -51, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 107, 0], [0, 0, 0, 0, 0, 835, 0, 0], [0, 0, 0, 85, 0, 0, 0, 0], [0, 0, 489, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 64], [790, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -215, 0], [0, 0, 0, 0, 0, 21, 0, 0], [0, 0, 0, -130, 0, 0, 0, 0], [0, -405, 0, 0, 0, 0, 0, 0], [-142, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 111, 0, 0], [0, 0, -339, 0, 0, 0, 0, 0], [0, -54, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 385, 0], [0, 0, 0, 0, -135, 0, 0, 0], [0, 0, 0, 0, 0, -535, 0, 0], [0, 0, -661, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 311, 0, 0], [0, 0, 0, -75, 0, 0, 0, 0], [0, 0, 0, 0, -110, 0, 0, 0], [0, 0, -1455, 0, 0, 0, 0, 0], [0, 345, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -159, 0, 0], [0, 0, 0, -64, 0, 0, 0, 0], [0, 0, 0, 0, -159, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 70], [0, 0, 0, 0, 0, 455, 0, 0], [0, 0, 641, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -50], [0, 85, 0, 0, 0, 0, 0, 0], [-2872, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 263, 0], [0, 0, 0, 169, 0, 0, 0, 0], [0, 255, 0, 0, 0, 0, 0, 0], [0, 0, 0, -5, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 49], [0, 0, 0, 0, 0, 719, 0, 0], [0, 0, 0, 0, -30, 0, 0, 0], [0, 0, -855, 0, 0, 0, 0, 0], [0, -355, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -609, 0, 0], [0, 0, 0, -14, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 91], [0, -50, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, -279, 0, 0, 0], [0, 0, -1711, 0, 0, 0, 0, 0], [0, -5, 0, 0, 0, 0, 0, 0], [1968, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -45, 0], [0, 0, 0, -26, 0, 0, 0, 0], [0, 0, 0, 0, -255, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 80], [0, 0, 0, 0, 0, 0, -275, 0], [-510, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, -1001, 0, 0], [0, 0, 0, -30, 0, 0, 0, 0], [0, 0, 0, 0, -135, 0, 0, 0], [0, 0, 0, 0, 194, 0, 0, 0], [0, 0, 725, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 71], [0, 0, 0, 0, 0, 0, 425, 0], [0, 0, -231, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -45], [0, 0, 0, 0, 0, 0, 503, 0], [0, 0, 0, 0, 270, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 70], [-1280, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 965, 0, 0], [-1360, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 681, 0, 0], [0, 0, 0, -165, 0, 0, 0, 0], [0, 0, 0, 0, 190, 0, 0, 0], [1802, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 96, 0, 0, 0, 0], [0, -295, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 1185, 0, 0], [0, 339, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 270, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, -135], [0, 60, 0, 0, 0, 0, 0, 0], [160, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -193, 0], [0, 0, 0, 0, 0, -435, 0, 0], [0, 0, -209, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -95, 0], [0, 0, 0, 20, 0, 0, 0, 0], [0, 0, -1505, 0, 0, 0, 0, 0], [0, 0, 0, -74, 0, 0, 0, 0], [0, 0, 0, 0, -166, 0, 0, 0], [0, 0, 0, 0, 0, 0, -225, 0], [0, 0, 0, 0, 0, -495, 0, 0], [0, 0, 0, 0, 0, 0, 315, 0], [0, 0, 0, -61, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 0, 15], [-2950, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, -377, 0], [0, 0, 0, 0, 0, 375, 0, 0]]
